--[[
    AutoHideCorpses - By wired420 @ redguides

    Automatically hides corpses based on certain criteria.
]]
--
local ahc_version = '1.03'
local mq = require("mq")
local ui = require("ImGui")
local LIP = require('lib/LIP')

local _terminate = false
local _initialized = false
local _enabled = false
local _needsave = false
local config_dir = mq.TLO.MacroQuest.Path():gsub('\\', '/')
local ahc_settings_file = '/config/AutoHideCorpses.ini'
local ahc_settings_path = config_dir .. ahc_settings_file
local ahc_npc_kills = 0
local ahc_settings = { ['ahc_corpse'] = { number = 0 },['ahc_kills'] = { number = 0 },['ahc_timer'] = { number = 0 } }
local ahc_old_settings

local ahc_time = mq.TLO.Time.SecondsSinceMidnight() + (ahc_settings['ahc_timer'].number * 60)

local function file_exists(path)
    local f = io.open(path, "r")
    if f ~= nil then
        io.close(f)
        return true
    else
        return false
    end
end

function table.copy(t)
    local t2 = {};
    for k, v in pairs(t) do
        if type(v) == "table" then
            t2[k] = table.copy(v);
        else
            t2[k] = v;
        end
    end
    return t2;
end

local SaveSettings = function()
    LIP.save(ahc_settings_path, ahc_settings)
end

local LoadSettings = function()
    if file_exists(ahc_settings_path) then
        ahc_settings = LIP.load(ahc_settings_path)
    else
        ahc_settings = { ['ahc_corpse'] = { number = 20 },['ahc_kills'] = { number = 30 },['ahc_timer'] = { number = 0 } }
        SaveSettings()
    end
end

local function logz(s, ...)
    print('\ar[\atAutoHideCorpses\ar]\aw ' .. string.format(s, ...))
end

logz('\ar[\awVersion\a-w:\aw %s \aw- By\a-w:\aw wired420\ar]\aw', ahc_version)

local function ahc_command()
    if Open then
        Open = false
    else
        Open = true
    end
end

local function show_help_marker(s)
    ImGui.SameLine()
    ImGui.TextDisabled('(?)')
    if ImGui.IsItemHovered(ImGuiHoveredFlags.None) then
        ImGui.BeginTooltip()
        ImGui.PushTextWrapPos(ImGui.GetFontSize() * 35.0)
        ImGui.TextUnformatted(s)
        ImGui.PopTextWrapPos()
        ImGui.EndTooltip()
    end
end

local function render_ahc_ui()
    if Open then
        local WindowFlags = bit32.bor(ImGuiWindowFlags.AlwaysAutoResize)
        Open, DrawUI = ui.Begin('Auto Hide Corpses', Open, WindowFlags)
        if DrawUI then
            _enabled = ui.Checkbox('Enable', _enabled)
            show_help_marker('Enable or disable the entire script.')
            ui.Spacing()
            ui.Separator()
            ui.Spacing()

            ui.Text('Hide Every "X" kills (0 to disable)')
            ahc_settings['ahc_kills'].number = ui.SliderInt('##SliderKillsNumber', ahc_settings['ahc_kills'].number, 0,
                99,
                '%d')

            ui.Spacing()

            ui.Text('Hide At "X" Corpses (0 to disable)')
            ahc_settings['ahc_corpse'].number = ui.SliderInt('##SliderCorpseNumber', ahc_settings['ahc_corpse'].number, 0,
                99,
                '%d')

            ui.Spacing()

            ui.Text('Hide Every "X" Minutes (0 to disable)')
            ahc_settings['ahc_timer'].number = ui.SliderInt('##SliderTimerNumber', ahc_settings['ahc_timer'].number, 0,
                60,
                '%d')

            ui.Spacing()
        end
        ui.End()
    end
end

local function ahc_hide_corpses()
    if mq.TLO.Plugin('MQ2DanNet').IsLoaded then
        mq.cmd('/dgga /hidec all')
        mq.delay(250)
        mq.cmd('/dgga /hidec looted')
    else
        mq.cmd('/hidec all')
        mq.delay(250)
        mq.cmd('/hidec looted')
    end
end

local function process_npc_kill()
    if _enabled then
        if ahc_settings['ahc_kills'].number > 0 then
            ahc_npc_kills = ahc_npc_kills + 1
            if ahc_npc_kills >= ahc_settings['ahc_kills'].number then
                logz('Reached %d kills. Hiding corpses.', ahc_npc_kills)
                ahc_npc_kills = 0
                ahc_hide_corpses()
            end
        end
        if ahc_settings['ahc_corpse'].number > 0 then
            local corpse_number = mq.TLO.SpawnCount('corpse radius 100')()
            if corpse_number >= ahc_settings['ahc_corpse'].number then
                logz('Detected %d corpses. Hiding Corpses', corpse_number)
                ahc_hide_corpses()
            end
        end
    end
end

mq.event('KilledNPC', 'You have slain #1#!', process_npc_kill)
mq.event('GroupKilledNPC', '#1# has been slain by #2#!', process_npc_kill)

while not _terminate
do
    if not _initialized then
        LoadSettings()
        ahc_old_settings = table.copy(ahc_settings)
        mq.imgui.init('autohidecorpses', render_ahc_ui)
        mq.bind('/ahc', ahc_command)
        Open = true
        _initialized = true;
        logz('Startup Complete')
    else
        if ahc_settings['ahc_timer'].number > 0 and _enabled then
            if mq.TLO.Time.SecondsSinceMidnight() >= ahc_time then
                ahc_time = mq.TLO.Time.SecondsSinceMidnight() + (ahc_settings['ahc_timer'].number * 60)
                logz('%d minutes elasped. Hiding Corpses', ahc_settings['ahc_timer'].number)
                ahc_hide_corpses()
            end
        end
    end

    for k, v in pairs(ahc_settings) do
        if v.number ~= ahc_old_settings[k].number then
            ahc_old_settings = table.copy(ahc_settings)
            _needsave = true
        end
    end

    if _needsave then
        SaveSettings()
        _needsave = false
    end

    mq.doevents()
    mq.delay(1)
end
if _terminate then
    mq.unbind('/ahc')
end
